unit BrowseDialog;
{*******************************************************************************
*  TBrowseDialog                                                               *
********************************************************************************
* Author: Raoul Snyman                                                         *
* ---------------------------------------------------------------------------- *
* E-Mail: components@saturnlaboratories.gq.nu                                  *
* ---------------------------------------------------------------------------- *
* Copyright: 2002 Saturn Laboratories, All rights reserved.                   *
* ---------------------------------------------------------------------------- *
* Description: A simple component to execute the "Browse For Folder" dialog in *
*              Windows.                                                        *
********************************************************************************
* This component is FREEWARE.                                                  *
* It may be used for commercial purposes on the condition that you give me     *
* credit (i.e. place it in your credits list).                                 *
* If used in freeware, it's not necessary to give me credit, although it would *
* be appreciated.                                                              *
* If you modify this code, please send me an e-mail with a copy of the code    *
* attached, letting me know what it was that you changed/modified/added.       *
********************************************************************************
* PROCEDURES IN THIS COMPONENT:                                                *
* ---------------------------------------------------------------------------- *
* Execute: This executes the browse dialog and returns true on successful and  *
*          false otherwise.                                                    *
********************************************************************************
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShlObj;

type
  TBrowseDialog = class(TComponent)
  private
    { Private declarations }
    FFolder: string;
    FTitle: string;
    FCaptionText: string;
    FBrowseInfo: TBrowseInfo;
  public
    { Public declarations }
    function Execute: boolean;
  published
    { Published declarations }
    property Folder: string read FFolder write FFolder;
    property Title: string read FTitle write FTitle;
    property CaptionText: string read FCaptionText write FCaptionText;
  end;

procedure Register;

implementation

{$R *.DCR}

procedure Register;
begin
 RegisterComponents('Dialogs', [TBrowseDialog]);
end;

function BrowseCallback(Wnd: HWND; uMsg: UINT; lParam, lpData: LPARAM): Integer stdcall;
begin
 Result := 0;
 if uMsg = BFFM_INITIALIZED
  then
   with TBrowseDialog(lpData) do
    begin
     SendMessage(Wnd, BFFM_SETSTATUSTEXT, 1, Longint(PChar(Folder)));
     if Length(Folder) > 0
      then SendMessage(Wnd, BFFM_SetSelection, 1, Longint(PChar(Folder)));
    end;
end;

function TBrowseDialog.Execute: boolean;
var Buffer: PChar;
    List: PItemIDList;
begin
 GetMem(Buffer, MAX_PATH);
 with FBrowseInfo do
  begin
   hwndOwner := Application.Handle;
   pidlRoot := nil;
   lpszTitle := PChar(FTitle);
   ulFlags := BIF_RETURNONLYFSDIRS + BIF_STATUSTEXT;
   lpfn := BrowseCallback;
   lParam := Longint(Self);
  end;
 list := ShBrowseForFolder(FBrowseInfo);
 if not (List = nil)
  then
   begin
    SHGetPathFromIDList(List, Buffer);
    FFolder := Buffer;
    Execute := True;
   end
  else Execute := False;
 FreeMem(Buffer);
end;

end.
